#include "x2_inc_switches"

// Get the PC, regardless of event
object GetPC();

// Get the item, regardless of event
object GetItem();

// Get the target of the item, regardless of event
object GetTarget();

// Exit if any event fires other than Activate
int ExitNotActivate();

// Exit if any event fires other than Equip
int ExitNotEquip();

// Exit if any event fires other than UnEquip
int ExitNotUnEquip();

// Exit if any event fires other than OnHitCast
int ExitNotOnHitCast();

// Exit if any event fires other than Acquire
int ExitNotAcquire();

// Exit if any event fires other than UnAcquire
int ExitNotUnAcquire();

// Exit if any event fires other than SpellCastAt
int ExitNotSpellCastAt();

// Exit if this is the Activate Event calling
int ExitActivate();

// Exit if this is the Equip Event calling
int ExitEquip();

// Exit if this is the UnEquip Event calling
int ExitUnEquip();

// Exit if this is the OnHitCast Event calling
int ExitOnHitCast();

// Exit if this is the Acquire Event calling
int ExitAcquire();

// Exit if this is the UnAcquire Event calling
int ExitUnAcquire();

// Exit if this is the SpellCastAt Event calling
int ExitSpellCastAt();

object GetPC()
{
  switch(GetUserDefinedItemEventNumber())
  {
  case X2_ITEM_EVENT_ACTIVATE: return GetItemActivator();
  case X2_ITEM_EVENT_EQUIP: return GetPCItemLastEquippedBy();
  case X2_ITEM_EVENT_UNEQUIP: return GetPCItemLastUnequippedBy();
  case X2_ITEM_EVENT_ONHITCAST: return OBJECT_SELF;
  case X2_ITEM_EVENT_ACQUIRE: return GetModuleItemAcquiredBy();
  case X2_ITEM_EVENT_UNACQUIRE: return GetModuleItemLostBy();
  case X2_ITEM_EVENT_SPELLCAST_AT: return OBJECT_SELF;
  }
  return OBJECT_INVALID;
}

object GetItem()
{
  switch(GetUserDefinedItemEventNumber())
  {
  case X2_ITEM_EVENT_ACTIVATE: return GetItemActivated();
  case X2_ITEM_EVENT_EQUIP: return GetPCItemLastEquipped();
  case X2_ITEM_EVENT_UNEQUIP: return GetPCItemLastUnequipped();
  case X2_ITEM_EVENT_ONHITCAST: return GetSpellCastItem();
  case X2_ITEM_EVENT_ACQUIRE: return GetModuleItemAcquired();
  case X2_ITEM_EVENT_UNACQUIRE: return GetModuleItemLost();
  case X2_ITEM_EVENT_SPELLCAST_AT: return GetSpellTargetObject();
  }
  return OBJECT_INVALID;
}

object GetTarget()
{
  switch(GetUserDefinedItemEventNumber())
  {
  case X2_ITEM_EVENT_ACTIVATE: return GetItemActivatedTarget();
  case X2_ITEM_EVENT_ONHITCAST: return GetSpellTargetObject();
  }
  return OBJECT_INVALID;
}

int ExitNotActivate() { if (GetUserDefinedItemEventNumber() != X2_ITEM_EVENT_ACTIVATE) return TRUE; return FALSE; }
int ExitNotEquip() { if (GetUserDefinedItemEventNumber() != X2_ITEM_EVENT_EQUIP) return TRUE; return FALSE; }
int ExitNotUnEquip() { if (GetUserDefinedItemEventNumber() != X2_ITEM_EVENT_UNEQUIP) return TRUE; return FALSE; }
int ExitNotOnHitCast() { if (GetUserDefinedItemEventNumber() != X2_ITEM_EVENT_ONHITCAST) return TRUE; return FALSE; }
int ExitNotAcquire() { if (GetUserDefinedItemEventNumber() != X2_ITEM_EVENT_ACQUIRE) return TRUE; return FALSE; }
int ExitNotUnAcquire() { if (GetUserDefinedItemEventNumber() != X2_ITEM_EVENT_UNACQUIRE) return TRUE; return FALSE; }
int ExitNotSpellCastAt() { if (GetUserDefinedItemEventNumber() != X2_ITEM_EVENT_SPELLCAST_AT) return TRUE; return FALSE; }

int ExitActivate() { if (GetUserDefinedItemEventNumber() == X2_ITEM_EVENT_ACTIVATE) return TRUE; return FALSE; }
int ExitEquip() { if (GetUserDefinedItemEventNumber() == X2_ITEM_EVENT_EQUIP) return TRUE; return FALSE; }
int ExitUnEquip() { if (GetUserDefinedItemEventNumber() == X2_ITEM_EVENT_UNEQUIP) return TRUE; return FALSE; }
int ExitOnHitCast() { if (GetUserDefinedItemEventNumber() == X2_ITEM_EVENT_ONHITCAST) return TRUE; return FALSE; }
int ExitAcquire() { if (GetUserDefinedItemEventNumber() == X2_ITEM_EVENT_ACQUIRE) return TRUE; return FALSE; }
int ExitUnAcquire() { if (GetUserDefinedItemEventNumber() == X2_ITEM_EVENT_UNACQUIRE) return TRUE; return FALSE; }
int ExitSpellCastAt() { if (GetUserDefinedItemEventNumber() == X2_ITEM_EVENT_SPELLCAST_AT) return TRUE; return FALSE; }

